@@ -20,7 +20,7 @@ package MongoDB;
 # ABSTRACT: Official MongoDB Driver for Perl
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 # regexp_pattern was unavailable before 5.10, had to be exported to load the
 # function implementation on 5.10, and was automatically available in 5.10.1
@@ -57,7 +57,7 @@ if ( $] eq '5.010' ) {
 #pod If an error occurs, a L<MongoDB::Error> object will be thrown.
 #pod
 #pod B<NOTE>: To connect to a replica set, a replica set name must be provided.
-#pod For example, if the set name is "setA":
+#pod For example, if the set name is C<"setA">:
 #pod
 #pod     $client = MongoDB->connect("mongodb://example.com/?replicaSet=setA");
 #pod
@@ -83,39 +83,12 @@ MongoDB - Official MongoDB Driver for Perl
 
 =head1 VERSION
 
-version v1.999.1
+version v2.1.0
 
-This is the Beta 1 release for v2.0.0.
-
-=head1 BETA RELEASE
-
-B<NOTE:> The v1.999.x versions are development releases in advance of the
-MongoDB Perl Driver v2.0.0. They are available for evaluation and testing
+B<NOTE:> The v2.1.x versions are development releases in advance of the
+MongoDB Perl Driver v2.2.0. They are available for evaluation and testing
 and should not be used in production.
 
-Details on incompatible changes and how to upgrade applications may be
-found in L<MongoDB::Upgrading>.
-
-This development release includes several significant changes since Alpha
-1, including:
-
-=over
-
-=item *
-
-The API for multi-document ACID transactions is now available.  See
-L<MongoDB::ClientSession> for details. (Requires MongoDB 4.0)
-
-=item *
-
-Change streams via the C<watch> method are now available at both single
-databases and client (all databases) levels. (Requires MongoDB 4.0)
-
-=back
-
-As of the Beta release, the driver is feature complete for MongoDB 4.0
-support.
-
 =head1 SYNOPSIS
 
     use MongoDB;
@@ -175,21 +148,26 @@ L<MongoDB community website|http://www.mongodb.org/>.
 
 =head1 USAGE
 
-The MongoDB driver is organized into a set of classes representing different
-levels of abstraction and functionality.
+The MongoDB driver is organized into a set of classes representing
+different levels of abstraction and functionality.
 
-As a user, you first create and configure a L<MongoDB::MongoClient> object to
-connect to a MongoDB deployment.  From that client object, you can get
-a L<MongoDB::Database> object for interacting with a specific database.
+As a user, you first create and configure a L<MongoDB::MongoClient> object
+to connect to a MongoDB deployment.  From that client object, you can get a
+L<MongoDB::Database> object for interacting with a specific database.
 
-From a database object, you can get a L<MongoDB::Collection> object for CRUD
-operations on that specific collection, or a L<MongoDB::GridFS> object for
-working with an abstract file system hosted on the database.  Each of those
-classes may return other objects for specific features or functions.
+From a database object, you can get a L<MongoDB::Collection> object for
+CRUD operations on that specific collection, or a L<MongoDB::GridFSBucket>
+object for working with an abstract file system hosted on the database.
+Each of those classes may return other objects for specific features or
+functions.
 
 See the documentation of those classes for more details or the
 L<MongoDB Perl Driver Tutorial|MongoDB::Tutorial> for an example.
 
+L<MongoDB::ClientSession> objects are generated from a
+L<MongoDB::MongoClient> and allow for advanced consistency options, like
+causal-consistency and transactions.
+
 =head2 Error handling
 
 Unless otherwise documented, errors result in fatal exceptions.  See
@@ -213,7 +191,7 @@ for L<MongoDB::MongoClient::new|MongoDB::MongoClient/ATTRIBUTES>.
 If an error occurs, a L<MongoDB::Error> object will be thrown.
 
 B<NOTE>: To connect to a replica set, a replica set name must be provided.
-For example, if the set name is "setA":
+For example, if the set name is C<"setA">:
 
     $client = MongoDB->connect("mongodb://example.com/?replicaSet=setA");
 
@@ -308,7 +286,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Andrew Page Andrey Khozov Ashley Willis Ask Bjørn Hansen Bernard Gorman Brendan W. McAdams Brian Moss Casey Rojas Christian Sturm Walde Colin Cyr Danny Raetzsch David Morrison Nadle Steinbrunner Storch diegok D. Ilmari Mannsåker Eric Daniels Gerard Goossen Glenn Fowler Graham Barr Hao Wu Harish Upadhyayula Jason Carey Toffaletti Johann Rolschewski John A. Kunze Joseph Harnish Josh Matthews Joshua Juran J. Stewart Kamil Slowikowski Ken Williams Matthew Shopsin Michael Langner Rotmanov Mike Dirolf Mohammad S Anwar Nickola Trupcheff Nigel Gregoire Niko Tyni Nuno Carvalho Orlando Vazquez Othello Maurer Pan Fan Pavel Denisov Rahul Dhodapkar Robert Sedlacek (Shadowcat Systems Ltd) Robin Lee Roman Yerin Ronald J Kimball Ryan Chipman Slaven Rezic Stephen Oberholtzer Steve Sanbeg Stuart Watt Thomas Bloor Uwe Voelker Whitney Jackson Xavier Guimard Xtreak Zhihong Zhang
+=for stopwords Andrew Page Andrey Khozov Ashley Willis Ask Bjørn Hansen Bernard Gorman Brendan W. McAdams Brian Moss Casey Rojas Christian Sturm Walde Colin Cyr Danny Raetzsch David Morrison Nadle Steinbrunner Storch diegok D. Ilmari Mannsåker Eric Daniels Finn Kempers (Shadowcat Systems Ltd) Gerard Goossen Glenn Fowler Graham Barr Hao Wu Harish Upadhyayula Jason Carey Toffaletti Johann Rolschewski John A. Kunze Joseph Harnish Josh Matthews Joshua Juran J. Stewart Kamil Slowikowski Ken Williams Matthew Shopsin Matt S Trout Michael Langner Rotmanov Mike Dirolf Mohammad Anwar Nickola Trupcheff Nigel Gregoire Niko Tyni Nuno Carvalho Orlando Vazquez Othello Maurer Pan Fan Pavel Denisov Rahul Dhodapkar Robert Sedlacek Robin Lee Roman Yerin Ronald J Kimball Ryan Chipman Slaven Rezic Stephen Oberholtzer Steve Sanbeg Stuart Watt Thomas Bloor Tobias Leich Uwe Voelker Wallace Reis Wan Bachtiar Whitney Jackson Xavier Guimard Xtreak Zhihong Zhang
 
 =over 4
 
@@ -394,6 +372,10 @@ Eric Daniels <eric.daniels@mongodb.com>
 
 =item *
 
+Finn Kempers (Shadowcat Systems Ltd) <toyou1995@gmail.com>
+
+=item *
+
 Gerard Goossen <gerard@ggoossen.net>
 
 =item *
@@ -458,6 +440,10 @@ Matthew Shopsin <matt.shopsin@mongodb.com>
 
 =item *
 
+Matt S Trout <mst@shadowcat.co.uk>
+
+=item *
+
 Michael Langner <langner@fch.de>
 
 =item *
@@ -534,6 +520,10 @@ Slaven Rezic <slaven.rezic@idealo.de>
 
 =item *
 
+Slaven Rezic <srezic@cpan.org>
+
+=item *
+
 Stephen Oberholtzer <stevie@qrpff.net>
 
 =item *
@@ -550,10 +540,22 @@ Thomas Bloor (Shadowcat Systems Ltd) <tbsliver@cpan.org>
 
 =item *
 
+Tobias Leich <email@froggs.de>
+
+=item *
+
 Uwe Voelker <uwe.voelker@xing.com>
 
 =item *
 
+Wallace Reis <wallace@reis.me>
+
+=item *
+
+Wan Bachtiar <sindbach@gmail.com>
+
+=item *
+
 Whitney Jackson <whjackson@gmail.com>
 
 =item *
@@ -572,7 +574,7 @@ Zhihong Zhang <zzh_621@yahoo.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by MongoDB, Inc.
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
