@@ -19,7 +19,7 @@ package MongoDB::BulkWriteView;
 # ABSTRACT: Bulk write operations against a query document
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
 
@@ -121,7 +121,6 @@ sub _update {
     }
 
     $self->_bulk->_retryable( 0 ) if $method eq 'update_many';
-    $self->_bulk->_retryable( 0 ) if $method eq 'delete_many';
 
     my $update = {
         q      => $self->_query,
@@ -140,6 +139,7 @@ sub _update {
 
 sub delete_many {
     my ($self) = @_;
+    $self->_bulk->_retryable( 0 );
     $self->_enqueue_write(
         [
             delete => {
@@ -182,7 +182,7 @@ MongoDB::BulkWriteView - Bulk write operations against a query document
 
 =head1 VERSION
 
-version v1.999.1
+version v2.1.0
 
 =head1 SYNOPSIS
 
@@ -326,7 +326,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by MongoDB, Inc.
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
