@@ -19,10 +19,9 @@ package MongoDB::ChangeStream;
 # ABSTRACT: A stream providing update information for collections.
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
-use Try::Tiny;
 use MongoDB::Cursor;
 use MongoDB::Op::_ChangeStream;
 use MongoDB::Error;
@@ -207,12 +206,11 @@ sub next {
     my $change;
     my $retried;
     while (1) {
-        last if try {
+        last if eval {
             $change = $self->_result->next;
-            1 # successfully fetched result
-        }
-        catch {
-            my $error = $_;
+            1; # successfully fetched result
+        } or do {
+            my $error = $@ || "Unknown error";
             if (
                 not($retried)
                 and $error->$_isa('MongoDB::Error')
@@ -224,7 +222,7 @@ sub next {
             else {
                 die $error;
             }
-            0 # failed, cursor was rebuilt
+            0; # failed, cursor was rebuilt
         };
     }
 
@@ -259,7 +257,7 @@ MongoDB::ChangeStream - A stream providing update information for collections.
 
 =head1 VERSION
 
-version v1.999.1
+version v2.1.0
 
 =head1 SYNOPSIS
 
@@ -325,7 +323,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by MongoDB, Inc.
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
