@@ -16,10 +16,10 @@ use strict;
 use warnings;
 package MongoDB::ClientSession;
 
-# ABSTRACT: MongoDB session management
+# ABSTRACT: MongoDB session and transaction management
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use MongoDB::Error;
 
@@ -212,7 +212,7 @@ sub get_latest_cluster_time {
 #pod current cluster time, then the session will be updated to this provided value.
 #pod
 #pod Setting the C<$clusterTime> with a manually crafted value may cause a server
-#pod error. It is reccomended to only use C<$clusterTime> values retrieved from
+#pod error. It is recommended to only use C<$clusterTime> values retrieved from
 #pod database calls.
 #pod
 #pod =cut
@@ -244,7 +244,7 @@ sub advance_cluster_time {
 #pod updated to this provided value.
 #pod
 #pod Setting C<operation_time> with a manually crafted value may cause a server
-#pod error. It is recommended to only use an C<operation_time> retreived from
+#pod error. It is recommended to only use an C<operation_time> retrieved from
 #pod another session or directly from a database call.
 #pod
 #pod =cut
@@ -273,7 +273,15 @@ sub _in_transaction_state {
 
 #pod =method start_transaction
 #pod
-#pod Start a transaction in this session. Takes a hashref of options which can contain the following options:
+#pod     $session->start_transaction;
+#pod     $session->start_transaction( $options );
+#pod
+#pod Start a transaction in this session.  If a transaction is already in
+#pod progress or if the driver can detect that the client is connected to a
+#pod topology that does not support transactions, this method will throw an
+#pod error.
+#pod
+#pod A hash reference of options may be provided. Valid keys include:
 #pod
 #pod =for :list
 #pod * C<readConcern> - The read concern to use for the first command in this
@@ -328,7 +336,32 @@ sub _increment_transaction_id {
 
 #pod =method commit_transaction
 #pod
-#pod Commit the current transaction. This will use the writeConcern set on this transaction.
+#pod     $session->commit_transaction;
+#pod
+#pod Commit the current transaction. This will use the writeConcern set on this
+#pod transaction.
+#pod
+#pod If called when no transaction is in progress, then this method will throw
+#pod an error.
+#pod
+#pod If the commit operation encounters an error, an error is thrown.  If the
+#pod error is a transient commit error, the error object will have a label
+#pod containing "UnknownTransactionCommitResult" as an element and the commit
+#pod operation can be retried.  This can be checked via the C<has_error_label>:
+#pod
+#pod     LOOP: {
+#pod         eval {
+#pod             $session->commit_transaction;
+#pod         };
+#pod         if ( my $error = $@ ) {
+#pod             if ( $error->has_error_label("UnknownTransactionCommitResult") ) {
+#pod                 redo LOOP;
+#pod             }
+#pod             else {
+#pod                 die $error;
+#pod             }
+#pod         }
+#pod     }
 #pod
 #pod =cut
 
@@ -352,7 +385,6 @@ sub commit_transaction {
     };
     if ( my $err = $@ ) {
         # catch and re-throw after retryable errors
-        # TODO maybe need better checking logic that theres actually an error code in output?
         my $err_code_name;
         my $err_code;
         if ( $err->can('result') ) {
@@ -392,7 +424,11 @@ sub commit_transaction {
 
 #pod =method abort_transaction
 #pod
-#pod Abort the current transaction. This will use the writeConcern set on this transaction.
+#pod     $session->abort_transaction;
+#pod
+#pod Aborts the current transaction.  If no transaction is in progress, then this
+#pod method will throw an error.  Otherwise, this method will suppress all other
+#pod errors (including network and database errors).
 #pod
 #pod =cut
 
@@ -435,7 +471,6 @@ sub _send_end_transaction_command {
         );
 
         my $result = $self->client->send_retryable_write_op( $op, 'force' );
-        # TODO This may be redundant after 918 is merged?
         $result->assert_no_write_concern_error;
     }
 
@@ -458,7 +493,10 @@ sub _maybe_apply_error_labels {
 #pod     $session->end_session;
 #pod
 #pod Close this particular session and release the session ID for reuse or
-#pod recycling.  Has no effect after calling for the first time.
+#pod recycling.  If a transaction is in progress, it will be aborted.  Has no
+#pod effect after calling for the first time.
+#pod
+#pod This will be called automatically by the object destructor.
 #pod
 #pod =cut
 
@@ -491,11 +529,11 @@ __END__
 
 =head1 NAME
 
-MongoDB::ClientSession - MongoDB session management
+MongoDB::ClientSession - MongoDB session and transaction management
 
 =head1 VERSION
 
-version v1.999.1
+version v2.1.0
 
 =head1 SYNOPSIS
 
@@ -504,6 +542,16 @@ version v1.999.1
     # use session in operations
     my $result = $collection->find( { id => 1 }, { session => $session } );
 
+    # use sessions for transactions
+    $session->start_transaction;
+    ...
+    if ( $ok ) {
+        $session->commit_transaction;
+    }
+    else {
+        $session->abort_transaction;
+    }
+
 =head1 DESCRIPTION
 
 This class encapsulates an active session for use with the current client.
@@ -535,6 +583,13 @@ Using a session across multiple threads is unsupported and unexpected issues
 and errors may occur. Note that the driver does not check for multi-threaded
 use.
 
+=head2 Transactions
+
+A session may be associated with at most one open transaction (on MongoDB
+4.0+).  For detailed instructions on how to use transactions with drivers,
+see the MongoDB manual page:
+L<Transactions|https://docs.mongodb.com/master/core/transactions>.
+
 =head1 ATTRIBUTES
 
 =head2 client
@@ -593,7 +648,7 @@ L</cluster_time>. If the cluster time provided is more recent than the sessions
 current cluster time, then the session will be updated to this provided value.
 
 Setting the C<$clusterTime> with a manually crafted value may cause a server
-error. It is reccomended to only use C<$clusterTime> values retrieved from
+error. It is recommended to only use C<$clusterTime> values retrieved from
 database calls.
 
 =head2 advance_operation_time
@@ -605,12 +660,20 @@ recent than the sessions current operation time, then the session will be
 updated to this provided value.
 
 Setting C<operation_time> with a manually crafted value may cause a server
-error. It is recommended to only use an C<operation_time> retreived from
+error. It is recommended to only use an C<operation_time> retrieved from
 another session or directly from a database call.
 
 =head2 start_transaction
 
-Start a transaction in this session. Takes a hashref of options which can contain the following options:
+    $session->start_transaction;
+    $session->start_transaction( $options );
+
+Start a transaction in this session.  If a transaction is already in
+progress or if the driver can detect that the client is connected to a
+topology that does not support transactions, this method will throw an
+error.
+
+A hash reference of options may be provided. Valid keys include:
 
 =over 4
 
@@ -630,18 +693,50 @@ C<readPreference> - The read preference to use for all read operations in this t
 
 =head2 commit_transaction
 
-Commit the current transaction. This will use the writeConcern set on this transaction.
+    $session->commit_transaction;
+
+Commit the current transaction. This will use the writeConcern set on this
+transaction.
+
+If called when no transaction is in progress, then this method will throw
+an error.
+
+If the commit operation encounters an error, an error is thrown.  If the
+error is a transient commit error, the error object will have a label
+containing "UnknownTransactionCommitResult" as an element and the commit
+operation can be retried.  This can be checked via the C<has_error_label>:
+
+    LOOP: {
+        eval {
+            $session->commit_transaction;
+        };
+        if ( my $error = $@ ) {
+            if ( $error->has_error_label("UnknownTransactionCommitResult") ) {
+                redo LOOP;
+            }
+            else {
+                die $error;
+            }
+        }
+    }
 
 =head2 abort_transaction
 
-Abort the current transaction. This will use the writeConcern set on this transaction.
+    $session->abort_transaction;
+
+Aborts the current transaction.  If no transaction is in progress, then this
+method will throw an error.  Otherwise, this method will suppress all other
+errors (including network and database errors).
 
 =head2 end_session
 
     $session->end_session;
 
 Close this particular session and release the session ID for reuse or
-recycling.  Has no effect after calling for the first time.
+recycling.  If a transaction is in progress, it will be aborted.  Has no
+effect after calling for the first time.
+
+This will be called automatically by the object destructor.
 
 =head1 AUTHORS
 
@@ -671,7 +766,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by MongoDB, Inc.
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
