@@ -20,7 +20,7 @@ package MongoDB::Database;
 # ABSTRACT: A MongoDB Database
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use MongoDB::CommandResult;
 use MongoDB::Error;
@@ -44,7 +44,6 @@ use Types::Standard qw(
 use Carp 'carp';
 use boolean;
 use Moo;
-use Try::Tiny;
 use namespace::clean -except => 'meta';
 
 has _client => (
@@ -149,6 +148,20 @@ has bson_codec => (
 # methods
 #--------------------------------------------------------------------------#
 
+#pod =method client
+#pod
+#pod     $client = $db->client;
+#pod
+#pod Returns the L<MongoDB::MongoClient> object associated with this
+#pod object.
+#pod
+#pod =cut
+
+sub client {
+    my ($self) = @_;
+    return $self->_client;
+}
+
 #pod =method list_collections
 #pod
 #pod     $result = $coll->list_collections( $filter );
@@ -179,13 +192,16 @@ has bson_codec => (
 #pod * C<batchSize> – the number of documents to return per batch.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
+#pod * C<nameOnly> - query and return names of the collections only. Defaults to
+#pod   false. (Note, this will be ignored for servers before version 4.0)
 #pod * C<session> - the session to use for these operations. If not supplied, will
 #pod   use an implicit session. For more information see L<MongoDB::ClientSession>
 #pod
+#pod B<NOTE>: When using C<nameOnly>, the filter query must be empty or must only
+#pod query the C<name> field or else no documents will be found.
+#pod
 #pod =cut
 
-my $list_collections_args;
-
 sub list_collections {
     my ( $self, $filter, $options ) = @_;
     $filter  ||= {};
@@ -215,6 +231,7 @@ sub list_collections {
 #pod
 #pod     my @collections = $database->collection_names;
 #pod     my @collections = $database->collection_names( $filter );
+#pod     my @collections = $database->collection_names( $filter, $options );
 #pod
 #pod Returns the list of collections in this database.
 #pod
@@ -239,9 +256,16 @@ sub list_collections {
 #pod =cut
 
 sub collection_names {
-    my $self = shift;
+    my ( $self, $filter, $options ) = @_;
 
-    my $res = $self->list_collections( @_ );
+    $filter ||= {};
+    $options ||= {};
+
+    my @filter_keys = keys %$filter;
+    $options->{nameOnly} =
+      @filter_keys == 0 || ( @filter_keys == 1 && exists $filter->{name} );
+
+    my $res = $self->list_collections( $filter, $options );
 
     return map { $_->{name} } $res->all;
 }
@@ -422,7 +446,32 @@ sub run_command {
     return $obj->output;
 }
 
-sub _aggregate {
+#pod =method aggregate
+#pod
+#pod Runs a query using the MongoDB 3.6+ aggregation framework and returns a
+#pod L<MongoDB::QueryResult> object.
+#pod
+#pod The first argument must be an array-ref of L<aggregation
+#pod pipeline|http://docs.mongodb.org/manual/core/aggregation-pipeline/> documents.
+#pod Each pipeline document must be a hash reference.
+#pod
+#pod The server supports several collection-less aggregation source stages like
+#pod C<$currentOp> and C<$listLocalSessions>.
+#pod
+#pod     $result = $database->aggregate( [
+#pod         {
+#pod             "\$currentOp" => {
+#pod                 allUsers => true,
+#pod             },
+#pod         },
+#pod     ] );
+#pod
+#pod See L<Aggregation|http://docs.mongodb.org/manual/aggregation/> in the MongoDB manual
+#pod for more information on how to construct aggregation queries.
+#pod
+#pod =cut
+
+sub aggregate {
     MongoDB::UsageError->throw("pipeline argument must be an array reference")
       unless ref( $_[1] ) eq 'ARRAY';
 
@@ -546,7 +595,7 @@ MongoDB::Database - A MongoDB Database
 
 =head1 VERSION
 
-version v1.999.1
+version v2.1.0
 
 =head1 SYNOPSIS
 
@@ -639,6 +688,13 @@ from a L<MongoDB::MongoClient> object.
 
 =head1 METHODS
 
+=head2 client
+
+    $client = $db->client;
+
+Returns the L<MongoDB::MongoClient> object associated with this
+object.
+
 =head2 list_collections
 
     $result = $coll->list_collections( $filter );
@@ -677,14 +733,22 @@ C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command
 
 =item *
 
+C<nameOnly> - query and return names of the collections only. Defaults to false. (Note, this will be ignored for servers before version 4.0)
+
+=item *
+
 C<session> - the session to use for these operations. If not supplied, will use an implicit session. For more information see L<MongoDB::ClientSession>
 
 =back
 
+B<NOTE>: When using C<nameOnly>, the filter query must be empty or must only
+query the C<name> field or else no documents will be found.
+
 =head2 collection_names
 
     my @collections = $database->collection_names;
     my @collections = $database->collection_names( $filter );
+    my @collections = $database->collection_names( $filter, $options );
 
 Returns the list of collections in this database.
 
@@ -808,6 +872,29 @@ There are a few examples of database commands in the
 L<MongoDB::Examples/"DATABASE COMMANDS"> section.  See also core documentation
 on database commands: L<http://dochub.mongodb.org/core/commands>.
 
+=head2 aggregate
+
+Runs a query using the MongoDB 3.6+ aggregation framework and returns a
+L<MongoDB::QueryResult> object.
+
+The first argument must be an array-ref of L<aggregation
+pipeline|http://docs.mongodb.org/manual/core/aggregation-pipeline/> documents.
+Each pipeline document must be a hash reference.
+
+The server supports several collection-less aggregation source stages like
+C<$currentOp> and C<$listLocalSessions>.
+
+    $result = $database->aggregate( [
+        {
+            "\$currentOp" => {
+                allUsers => true,
+            },
+        },
+    ] );
+
+See L<Aggregation|http://docs.mongodb.org/manual/aggregation/> in the MongoDB manual
+for more information on how to construct aggregation queries.
+
 =head2 watch
 
 Watches for changes on this database.
@@ -867,7 +954,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by MongoDB, Inc.
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
