@@ -23,7 +23,7 @@ package MongoDB::Error;
 
 use version;
 
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
 use Carp;
@@ -187,7 +187,7 @@ sub _is_retryable {
 
     return 1 if _check_is_retryable_message( $self->message );
 
-    if ( $self->$_isa( 'MongoDB::WriteConcernError' ) ) {
+    if ( $self->$_isa( 'MongoDB::WriteConcernError' ) && $self->result->$_can( 'output' ) ) {
       return 1 if _check_is_retryable_code( $self->result->output->{writeConcernError}{code} );
       return 1 if _check_is_retryable_message( $self->result->output->{writeConcernError}{message} );
     }
@@ -416,7 +416,7 @@ MongoDB::Error - MongoDB Driver Error classes
 
 =head1 VERSION
 
-version v1.999.1
+version v2.1.0
 
 =head1 SYNOPSIS
 
@@ -556,7 +556,7 @@ Attributes include:
 
 =item *
 
-result — response from a database command; this must impliement the C<last_errmsg> method
+result — response from a database command; this must implement the C<last_errmsg> method
 
 =item *
 
@@ -734,7 +734,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by MongoDB, Inc.
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
