@@ -19,7 +19,7 @@ package MongoDB::GridFSBucket::DownloadStream;
 # ABSTRACT: File handle abstraction for downloading
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
 use Types::Standard qw(
@@ -67,6 +67,7 @@ has file_doc => (
 has _buffer => (
     is  => 'rwp',
     isa => Str,
+    default => "",
 );
 
 has _chunk_n => (
@@ -247,7 +248,7 @@ sub _read_all {
 #pod * Calling close will also cause any tied file handles created for the
 #pod   stream to also close.
 #pod * C<close> will be automatically called when a stream object is destroyed.
-#pod * Calling C<close> repeately will warn.
+#pod * Calling C<close> repeatedly will warn.
 #pod
 #pod =cut
 
@@ -329,11 +330,9 @@ sub read {
         MongoDB::UsageError->throw(
             'Negative length passed to MongoDB::GridFSBucket::DownloadStream->read');
     }
-    my $bufflen = length $$buffref;
-
     $offset   ||= 0;
-    $bufflen  ||= 0;
     $$buffref ||= '';
+    my $bufflen = length $$buffref;
 
     $offset = max( 0, $bufflen + $offset ) if $offset < 0;
     if ( $offset > $bufflen ) {
@@ -437,7 +436,7 @@ MongoDB::GridFSBucket::DownloadStream - File handle abstraction for downloading
 
 =head1 VERSION
 
-version v1.999.1
+version v2.1.0
 
 =head1 SYNOPSIS
 
@@ -552,7 +551,7 @@ C<close> will be automatically called when a stream object is destroyed.
 
 =item *
 
-Calling C<close> repeately will warn.
+Calling C<close> repeatedly will warn.
 
 =back
 
@@ -616,7 +615,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by MongoDB, Inc.
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
