@@ -19,7 +19,7 @@ package MongoDB::IndexView;
 # ABSTRACT: Index management for a collection
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
 use MongoDB::Error;
@@ -67,7 +67,7 @@ has _bson_codec => (
 
 sub _build__bson_codec {
     my ($self) = @_;
-    return $self->collection->bson_codec;
+    return $self->collection->bson_codec->clone( ordered => 1 );
 }
 
 has _client => (
@@ -467,7 +467,7 @@ MongoDB::IndexView - Index management for a collection
 
 =head1 VERSION
 
-version v1.999.1
+version v2.1.0
 
 =head1 SYNOPSIS
 
@@ -716,7 +716,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by MongoDB, Inc.
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
