@@ -19,23 +19,24 @@ package MongoDB::MongoClient;
 # ABSTRACT: A connection to a MongoDB server or multi-server deployment
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
+use MongoDB::ClientSession;
 use MongoDB::Cursor;
 use MongoDB::Error;
 use MongoDB::Op::_Command;
 use MongoDB::Op::_FSyncUnlock;
+use MongoDB::ReadConcern;
 use MongoDB::ReadPreference;
 use MongoDB::WriteConcern;
-use MongoDB::ReadConcern;
-use MongoDB::ClientSession;
-use MongoDB::_SessionPool;
-use MongoDB::_Topology;
 use MongoDB::_Constants;
 use MongoDB::_Credential;
+use MongoDB::_Dispatcher;
+use MongoDB::_SessionPool;
+use MongoDB::_Topology;
 use MongoDB::_URI;
-use BSON 1.006005;
+use BSON 1.010001;
 use Digest::MD5;
 use UUID::URandom;
 use Tie::IxHash;
@@ -45,7 +46,6 @@ use Safe::Isa 1.000007;
 use Scalar::Util qw/reftype weaken/;
 use boolean;
 use Encode;
-use Try::Tiny;
 use MongoDB::_Types qw(
     ArrayOfHashRef
     AuthMechanism
@@ -381,7 +381,7 @@ sub _build_j {
     return $self->__uri_or_else(
         u => 'journal',
         e => 'j',
-        d => 0,
+        d => undef,
     );
 }
 
@@ -543,6 +543,40 @@ has port => (
     default => 27017,
 );
 
+#pod =attr read_concern_level
+#pod
+#pod The read concern level determines the consistency level required
+#pod of data being read.
+#pod
+#pod The default level is C<undef>, which means the server will use its configured
+#pod default.
+#pod
+#pod If the level is set to "local", reads will return the latest data a server has
+#pod locally.
+#pod
+#pod Additional levels are storage engine specific.  See L<Read
+#pod Concern|http://docs.mongodb.org/manual/search/?query=readConcern> in the MongoDB
+#pod documentation for more details.
+#pod
+#pod This may be set in a connection string with the the C<readConcernLevel> option.
+#pod
+#pod =cut
+
+has read_concern_level => (
+    is      => 'lazy',
+    isa     => Maybe [Str],
+    builder => '_build_read_concern_level',
+);
+
+sub _build_read_concern_level {
+    my ($self) = @_;
+    return $self->__uri_or_else(
+        u => 'readconcernlevel',
+        e => 'read_concern_level',
+        d => undef,
+    );
+}
+
 #pod =attr read_pref_mode
 #pod
 #pod The read preference mode determines which server types are candidates
@@ -640,6 +674,61 @@ sub _build_replica_set_name {
     );
 }
 
+#pod =attr retry_writes
+#pod
+#pod Whether the client should use retryable writes for supported commands. The
+#pod default value is false, which means that no write commands will be retried.
+#pod
+#pod If this is set to a true value, then commands which support retryable writes
+#pod will be retried on certain errors, such as C<not master> and C<node is
+#pod recovering> errors.
+#pod
+#pod This may be set in a connection string with the C<retryWrites> option.
+#pod
+#pod Note that this is only supported on MongoDB > 3.6 in Replica Set or Shard
+#pod Clusters, and will be ignored on other deployments.
+#pod
+#pod Unacknowledged write operations also do not support retryable writes, even when
+#pod retry_writes has been enabled.
+#pod
+#pod The supported single statement write operations are currently as follows:
+#pod
+#pod =for :list
+#pod * C<insert_one>
+#pod * C<update_one>
+#pod * C<replace_one>
+#pod * C<delete_one>
+#pod * C<find_one_and_delete>
+#pod * C<find_one_and_replace>
+#pod * C<find_one_and_update>
+#pod
+#pod The supported multi statement write operations are as follows:
+#pod
+#pod =for :list
+#pod * C<insert_many>
+#pod * C<bulk_write>
+#pod
+#pod The multi statement operations may be ether ordered or unordered. Note that for
+#pod C<bulk_write> operations, the request may not include update_many or
+#pod delete_many operations.
+#pod
+#pod =cut
+
+has retry_writes => (
+    is      => 'lazy',
+    isa     => Boolish,
+    builder => '_build_retry_writes',
+);
+
+sub _build_retry_writes {
+    my ( $self ) = @_;
+    return $self->__uri_or_else(
+        u => 'retrywrites',
+        e => 'retry_writes',
+        d => 0,
+    );
+}
+
 #pod =attr server_selection_timeout_ms
 #pod
 #pod This attribute specifies the amount of time in milliseconds to wait for a
@@ -705,6 +794,22 @@ sub _build_server_selection_try_once {
     );
 }
 
+#pod =attr server_selector
+#pod
+#pod Optional. This takes a function that augments the server selection rules.
+#pod The function takes as a parameter a list of server descriptions representing
+#pod the suitable servers for the read or write operation, and returns a list of
+#pod server descriptions that should still be considered suitable. Most users
+#pod should rely on the default server selection algorithm and should not need
+#pod to set this attribute.
+#pod
+#pod =cut
+
+has server_selector => (
+    is  => 'ro',
+    isa => Maybe[CodeRef],
+);
+
 #pod =attr socket_check_interval_ms
 #pod
 #pod If a socket to a server has not been used in this many milliseconds, an
@@ -744,6 +849,10 @@ sub _build_socket_check_interval_ms {
 #pod until the server replies or until the operating system TCP/IP stack
 #pod gives up.
 #pod
+#pod The driver automatically sets the TCP keepalive option when initializing the
+#pod socket. For keepalive related issues, check the MongoDB documentation for
+#pod L<Does TCP keepalive time affect MongoDB Deployments?|https://docs.mongodb.com/v3.2/faq/diagnostics/#does-tcp-keepalive-time-affect-mongodb-deployments>.
+#pod
 #pod A zero value polls the socket for available data and is thus likely to fail
 #pod except when talking to a local process (and perhaps even then).
 #pod
@@ -870,7 +979,7 @@ sub _build_username {
 #pod the server has received and processed the request. Older documentation may refer
 #pod to this as "fire-and-forget" mode.  This option is not recommended.
 #pod
-#pod =item * C<1> Acknowledged. This is the default. MongoClient will wait until the
+#pod =item * C<1> Acknowledged. MongoClient will wait until the
 #pod primary MongoDB acknowledges the write.
 #pod
 #pod =item * C<2> Replica acknowledged. MongoClient will wait until at least two
@@ -883,6 +992,8 @@ sub _build_username {
 #pod
 #pod =back
 #pod
+#pod If not set, the server default is used, which is typically "1".
+#pod
 #pod In MongoDB v2.0+, you can "tag" replica members. With "tagging" you can
 #pod specify a custom write concern For more information see L<Data Center
 #pod Awareness|http://docs.mongodb.org/manual/data-center-awareness/>
@@ -911,7 +1022,8 @@ sub _build_w {
 #pod The number of milliseconds an operation should wait for C<w> secondaries to
 #pod replicate it.
 #pod
-#pod Defaults to 1000 (1 second).
+#pod Defaults to 1000 (1 second). If you set this to undef, it could block indefinitely
+#pod (or until socket timeout is reached).
 #pod
 #pod See C<w> above for more information.
 #pod
@@ -921,7 +1033,7 @@ sub _build_w {
 
 has wtimeout => (
     is      => 'lazy',
-    isa     => Int,
+    isa     => Maybe[Int],
     builder => '_build_wtimeout',
 );
 
@@ -934,95 +1046,6 @@ sub _build_wtimeout {
     );
 }
 
-#pod =attr read_concern_level
-#pod
-#pod The read concern level determines the consistency level required
-#pod of data being read.
-#pod
-#pod The default level is C<undef>, which means the server will use its configured
-#pod default.
-#pod
-#pod If the level is set to "local", reads will return the latest data a server has
-#pod locally.
-#pod
-#pod Additional levels are storage engine specific.  See L<Read
-#pod Concern|http://docs.mongodb.org/manual/search/?query=readConcern> in the MongoDB
-#pod documentation for more details.
-#pod
-#pod This may be set in a connection string with the the C<readConcernLevel> option.
-#pod
-#pod =cut
-
-has read_concern_level => (
-    is      => 'lazy',
-    isa     => Maybe [Str],
-    builder => '_build_read_concern_level',
-);
-
-sub _build_read_concern_level {
-    my ($self) = @_;
-    return $self->__uri_or_else(
-        u => 'readconcernlevel',
-        e => 'read_concern_level',
-        d => undef,
-    );
-}
-
-#pod =attr retry_writes
-#pod
-#pod Whether the client should use retryable writes for supported commands. The
-#pod default value is false, which means that no write commands will be retried.
-#pod
-#pod If this is set to a true value, then commands which support retryable writes
-#pod will be retried on certain errors, such as C<not master> and C<node is
-#pod recovering> errors.
-#pod
-#pod This may be set in a connection string with the C<retryWrites> option.
-#pod
-#pod Note that this is only supported on MongoDB > 3.6 in Replica Set or Shard
-#pod Clusters, and will be ignored on other deployments.
-#pod
-#pod Unacknowledged write operations also do not support retryable writes, even when
-#pod retry_writes has been enabled.
-#pod
-#pod The supported single statement write operations are currently as follows:
-#pod
-#pod =for :list
-#pod * C<insert_one>
-#pod * C<update_one>
-#pod * C<replace_one>
-#pod * C<delete_one>
-#pod * C<find_one_and_delete>
-#pod * C<find_one_and_replace>
-#pod * C<find_one_and_update>
-#pod
-#pod The supported multi statement write operations are as follows:
-#pod
-#pod =for :list
-#pod * C<insert_many>
-#pod * C<bulk_write>
-#pod
-#pod The multi statment operations may be ether ordered or unordered. Note that for
-#pod C<bulk_write> operations, the request may not include update_many or
-#pod delete_many operations.
-#pod
-#pod =cut
-
-has retry_writes => (
-    is      => 'lazy',
-    isa     => Boolish,
-    builder => '_build_retry_writes',
-);
-
-sub _build_retry_writes {
-    my ( $self ) = @_;
-    return $self->__uri_or_else(
-        u => 'retrywrites',
-        e => 'retry_writes',
-        d => 0,
-    );
-}
-
 #--------------------------------------------------------------------------#
 # computed attributes - these are private and can't be set in the
 # constructor, but have a public accessor
@@ -1077,8 +1100,8 @@ sub _build__write_concern {
     return MongoDB::WriteConcern->new(
         # Must check for defined as w can be 0, and defaults to undef
         ( defined $self->w ? ( w        => $self->w )        : () ),
-        ( $self->wtimeout ? ( wtimeout => $self->wtimeout ) : () ),
-        ( $self->j        ? ( j        => $self->j )        : () ),
+        ( wtimeout => $self->wtimeout ),
+        ( defined $self->j        ? ( j        => $self->j )        : () ),
     );
 }
 
@@ -1188,6 +1211,8 @@ sub _build__topology {
         monitoring_callback => $self->monitoring_callback,
         compressors => $self->compressors,
         zlib_compression_level => $self->zlib_compression_level,
+        socket_check_interval_sec => $self->socket_check_interval_ms / 1000,
+        server_selector => $self->server_selector,
     );
 }
 
@@ -1232,13 +1257,44 @@ sub _build__uri {
     }
 }
 
+has _dispatcher => (
+    is       => 'lazy',
+    isa      => InstanceOf ['MongoDB::_Dispatcher'],
+    init_arg => undef,
+    builder  => '_build__dispatcher',
+    handles  => [
+        qw(
+          send_direct_op
+          send_primary_op
+          send_read_op
+          send_retryable_write_op
+          send_write_op
+          )
+    ],
+);
+
+sub _build__dispatcher {
+    my $self = shift;
+    return MongoDB::_Dispatcher->new(
+        topology     => $self->_topology,
+        retry_writes => $self->retry_writes,
+    );
+}
+
 has _server_session_pool => (
     is => 'lazy',
     isa => InstanceOf['MongoDB::_SessionPool'],
     init_arg => undef,
-    builder => sub { return MongoDB::_SessionPool->new( client => $_[0] ) },
+    builder => '_build__server_session_pool',
 );
 
+sub _build__server_session_pool {
+    my $self = shift;
+    return MongoDB::_SessionPool->new(
+        dispatcher => $self->_dispatcher,
+        topology   => $self->_topology,
+    );
+}
 
 #--------------------------------------------------------------------------#
 # Constructor customization
@@ -1389,14 +1445,15 @@ sub disconnect {
 #pod     $client->reconnect;
 #pod
 #pod This method closes all connections to the server, as if L</disconnect> were
-#pod called, and then immediately reconnects.  Use this after forking or spawning
-#pod off a new thread.
+#pod called, and then immediately reconnects.  It also clears the session
+#pod cache.  Use this after forking or spawning off a new thread.
 #pod
 #pod =cut
 
 sub reconnect {
     my ($self) = @_;
     $self->_topology->close_all_links;
+    $self->_server_session_pool->reset_pool;
     $self->_topology->scan_all_servers(1);
     return 1;
 }
@@ -1527,189 +1584,6 @@ sub send_admin_command {
     return $self->send_read_op( $op );
 }
 
-# Reset session state if we're outside an active transaction, otherwise set
-# that this transaction actually has operations
-sub _maybe_update_session_state {
-    my ( $self, $op ) = @_;
-    if ( defined $op->session && ! $op->session->_active_transaction ) {
-        $op->session->_set__transaction_state( TXN_NONE );
-    } elsif ( defined $op->session ) {
-        $op->session->_set__has_transaction_operations( 1 );
-    }
-}
-
-# op dispatcher written in highly optimized style
-sub send_direct_op {
-    my ( $self, $op, $address ) = @_;
-    my ( $link, $result );
-
-    $self->_maybe_update_session_state( $op );
-
-    ( $link = $self->{_topology}->get_specific_link($address) ), (
-        eval { ($result) = $op->execute($link); 1 } or do {
-            my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            if ( $err->$_isa("MongoDB::ConnectionError") ) {
-                $self->{_topology}->mark_server_unknown( $link->server, $err );
-            }
-            elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
-                $self->{_topology}->mark_server_unknown( $link->server, $err );
-                $self->{_topology}->mark_stale;
-            }
-            # regardless of cleanup, rethrow the error
-            WITH_ASSERTS ? ( confess $err ) : ( die $err );
-          }
-      ),
-      return $result;
-}
-
-# op dispatcher written in highly optimized style
-sub send_write_op {
-    my ( $self, $op ) = @_;
-    my ( $link, $result );
-
-    $self->_maybe_update_session_state( $op );
-
-    ( $link = $self->{_topology}->get_writable_link ), (
-        eval { ($result) = $self->_try_write_op_for_link( $link, $op ); 1 } or do {
-            my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            WITH_ASSERTS ? ( confess $err ) : ( die $err );
-          }
-      ),
-      return $result;
-}
-
-# Sometimes, seeing an op dispatched as "send_write_op" is confusing when
-# really, we're just insisting that it be sent only to a primary or
-# directly connected server.
-BEGIN {
-    no warnings 'once';
-    *send_primary_op = \&send_write_op;
-}
-
-sub send_retryable_write_op {
-    my ( $self, $op, $force ) = @_;
-
-    my $result;
-    my $link = $self->{_topology}->get_writable_link;
-
-    $self->_maybe_update_session_state( $op );
-
-    # Need to force to do a retryable write on a Transaction Commit or Abort. $force is an override for retry_writes, but theres no point trying that if the link doesnt support it anyway.
-    # This triggers on the following:
-    # * $force is not set to 'force'
-    #   (specifically for retrying writes in ending transaction operations)
-    # * retry writes is not enabled or the link doesnt support retryWrites
-    # * if an active transaction is starting or in progress
-    unless ( $link->supports_retryWrites
-        && ( $self->retry_writes || ( defined $force && $force eq 'force' ) )
-        && ( defined $op->session
-          && ! $op->session->_in_transaction_state( TXN_STARTING, TXN_IN_PROGRESS )
-        )
-    ) {
-        eval { ($result) = $self->_try_write_op_for_link( $link, $op ); 1 } or do {
-            my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            WITH_ASSERTS ? ( confess $err ) : ( die $err );
-        };
-        return $result;
-    }
-
-    # If we get this far and there is no session, then somethings gone really
-    # wrong, so probably not worth worrying about.
-
-    # increment transaction id before write, but otherwise is the same for both
-    # attempts. If not in a transaction, is a no-op
-    $op->session->_increment_transaction_id;
-    $op->retryable_write( 1 );
-
-    # attempt the op the first time
-    eval { ($result) = $self->_try_write_op_for_link( $link, $op ); 1 } or do {
-        my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-
-        # If the error is not retryable, then drop out
-        unless ( $err->$_call_if_can('_is_retryable') ) {
-            WITH_ASSERTS ? ( confess $err ) : ( die $err );
-        }
-
-        # Must check if error is retryable before getting the link, in case we
-        # get a 'no writable servers' error
-        my $retry_link = $self->{_topology}->get_writable_link;
-
-        # Rare chance that the new link is not retryable
-        unless ( $retry_link->supports_retryWrites ) {
-            WITH_ASSERTS ? ( confess $err ) : ( die $err );
-        }
-
-        # Second attempt
-        eval { ($result) = $self->_try_write_op_for_link( $retry_link, $op ); 1 } or do {
-            my $retry_err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            # Only network or not_master errors should propogate on second attempt
-            if ( $retry_err->$_isa("MongoDB::ConnectionError")
-              || $retry_err->$_isa("MongoDB::NotMasterError") ) {
-                WITH_ASSERTS ? ( confess $retry_err ) : ( die $retry_err );
-            }
-            # die with original error otherwise
-            WITH_ASSERTS ? ( confess $err ) : ( die $err );
-        };
-    };
-    # just in case this gets reused for some reason
-    $op->retryable_write( 0 );
-    return $result;
-}
-
-# op dispatcher written in highly optimized style
-sub _try_write_op_for_link {
-    my ( $self, $link, $op ) = @_;
-    my $result;
-    (
-        eval { ($result) = $op->execute($link, $self->{_topology}->type); 1 } or do {
-            my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            if ( $err->$_isa("MongoDB::ConnectionError") ) {
-                $self->{_topology}->mark_server_unknown( $link->server, $err );
-            }
-            elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
-                $self->{_topology}->mark_server_unknown( $link->server, $err );
-                $self->{_topology}->mark_stale;
-            }
-            # normal die here instead of assert, which is used later
-            die $err;
-        }
-    ),
-    return $result;
-}
-
-# op dispatcher written in highly optimized style
-sub send_read_op {
-    my ( $self, $op ) = @_;
-    my ( $link, $type, $result );
-
-    # Get transaction read preference if in a transaction.
-    if ( defined $op->session && $op->session->_active_transaction ) {
-        # Transactions may only read from primary in MongoDB 4.0, so get and
-        # check the read preference from the transaction settings as per
-        # transaction spec - see MongoDB::_TransactionOptions
-        $op->read_preference( $op->session->_get_transaction_read_preference );
-    }
-
-    $self->_maybe_update_session_state( $op );
-
-    ( $link = $self->{_topology}->get_readable_link( $op->read_preference ) ),
-      ( $type = $self->{_topology}->type ), (
-        eval { ($result) = $op->execute( $link, $type ); 1 } or do {
-            my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            if ( $err->$_isa("MongoDB::ConnectionError") ) {
-                $self->{_topology}->mark_server_unknown( $link->server, $err );
-            }
-            elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
-                $self->{_topology}->mark_server_unknown( $link->server, $err );
-                $self->{_topology}->mark_stale;
-            }
-            # regardless of cleanup, rethrow the error
-            WITH_ASSERTS ? ( confess $err ) : ( die $err );
-          }
-      ),
-      return $result;
-}
-
 #--------------------------------------------------------------------------#
 # database helper methods
 #--------------------------------------------------------------------------#
@@ -1739,17 +1613,18 @@ sub list_databases {
     my @databases;
     my $max_tries = 3;
     for my $try ( 1 .. $max_tries ) {
-        last if try {
+        last if eval {
             my $output = $self->send_admin_command([ listDatabases => 1, ( $args ? %$args : () ) ])->output;
             if (ref($output) eq 'HASH' && exists $output->{databases}) {
                 @databases = @{ $output->{databases} };
             }
             return 1;
-        } catch {
-            if ( $_->$_isa("MongoDB::DatabaseError" ) ) {
-                return if $_->result->output->{code} == CANT_OPEN_DB_IN_READ_LOCK() || $try < $max_tries;
+        } or do {
+            my $error = $@ || "Unknown error";
+            if ( $error->$_isa("MongoDB::DatabaseError" ) ) {
+                return if $error->result->output->{code} == CANT_OPEN_DB_IN_READ_LOCK() || $try < $max_tries;
             }
-            die $_;
+            die $error;
         };
     }
 
@@ -1980,6 +1855,12 @@ sub watch {
     );
 }
 
+sub _primary_server_version {
+    my $self = shift;
+    my $build = $self->send_admin_command( [ buildInfo => 1 ] )->output;
+    my ($version_str) = $build->{version} =~ m{^([0-9.]+)};
+    return version->parse("v$version_str");
+}
 
 1;
 
@@ -1995,7 +1876,7 @@ MongoDB::MongoClient - A connection to a MongoDB server or multi-server deployme
 
 =head1 VERSION
 
-version v1.999.1
+version v2.1.0
 
 =head1 SYNOPSIS
 
@@ -2281,6 +2162,23 @@ An empty password still requires a ":" character.
 If a network port is not specified as part of the C<host> attribute, this
 attribute provides the port to use.  It defaults to 27107.
 
+=head2 read_concern_level
+
+The read concern level determines the consistency level required
+of data being read.
+
+The default level is C<undef>, which means the server will use its configured
+default.
+
+If the level is set to "local", reads will return the latest data a server has
+locally.
+
+Additional levels are storage engine specific.  See L<Read
+Concern|http://docs.mongodb.org/manual/search/?query=readConcern> in the MongoDB
+documentation for more details.
+
+This may be set in a connection string with the the C<readConcernLevel> option.
+
 =head2 read_pref_mode
 
 The read preference mode determines which server types are candidates
@@ -2342,6 +2240,75 @@ names must match this or they will be removed from the topology.
 
 This may be set in a connection string with the C<replicaSet> option.
 
+=head2 retry_writes
+
+Whether the client should use retryable writes for supported commands. The
+default value is false, which means that no write commands will be retried.
+
+If this is set to a true value, then commands which support retryable writes
+will be retried on certain errors, such as C<not master> and C<node is
+recovering> errors.
+
+This may be set in a connection string with the C<retryWrites> option.
+
+Note that this is only supported on MongoDB > 3.6 in Replica Set or Shard
+Clusters, and will be ignored on other deployments.
+
+Unacknowledged write operations also do not support retryable writes, even when
+retry_writes has been enabled.
+
+The supported single statement write operations are currently as follows:
+
+=over 4
+
+=item *
+
+C<insert_one>
+
+=item *
+
+C<update_one>
+
+=item *
+
+C<replace_one>
+
+=item *
+
+C<delete_one>
+
+=item *
+
+C<find_one_and_delete>
+
+=item *
+
+C<find_one_and_replace>
+
+=item *
+
+C<find_one_and_update>
+
+=back
+
+The supported multi statement write operations are as follows:
+
+=over 4
+
+=item *
+
+C<insert_many>
+
+=item *
+
+C<bulk_write>
+
+=back
+
+The multi statement operations may be ether ordered or unordered. Note that for
+C<bulk_write> operations, the request may not include update_many or
+delete_many operations.
+
 =head2 server_selection_timeout_ms
 
 This attribute specifies the amount of time in milliseconds to wait for a
@@ -2373,6 +2340,15 @@ See L</SERVER SELECTION> for more details.
 This may be set in a connection string with the C<serverSelectionTryOnce>
 option.
 
+=head2 server_selector
+
+Optional. This takes a function that augments the server selection rules.
+The function takes as a parameter a list of server descriptions representing
+the suitable servers for the read or write operation, and returns a list of
+server descriptions that should still be considered suitable. Most users
+should rely on the default server selection algorithm and should not need
+to set this attribute.
+
 =head2 socket_check_interval_ms
 
 If a socket to a server has not been used in this many milliseconds, an
@@ -2395,6 +2371,10 @@ If set to a negative value, socket operations will block indefinitely
 until the server replies or until the operating system TCP/IP stack
 gives up.
 
+The driver automatically sets the TCP keepalive option when initializing the
+socket. For keepalive related issues, check the MongoDB documentation for
+L<Does TCP keepalive time affect MongoDB Deployments?|https://docs.mongodb.com/v3.2/faq/diagnostics/#does-tcp-keepalive-time-affect-mongodb-deployments>.
+
 A zero value polls the socket for available data and is thus likely to fail
 except when talking to a local process (and perhaps even then).
 
@@ -2465,7 +2445,7 @@ The client I<write concern>.
 the server has received and processed the request. Older documentation may refer
 to this as "fire-and-forget" mode.  This option is not recommended.
 
-=item * C<1> Acknowledged. This is the default. MongoClient will wait until the
+=item * C<1> Acknowledged. MongoClient will wait until the
 primary MongoDB acknowledges the write.
 
 =item * C<2> Replica acknowledged. MongoClient will wait until at least two
@@ -2478,6 +2458,8 @@ number for more replicas.
 
 =back
 
+If not set, the server default is used, which is typically "1".
+
 In MongoDB v2.0+, you can "tag" replica members. With "tagging" you can
 specify a custom write concern For more information see L<Data Center
 Awareness|http://docs.mongodb.org/manual/data-center-awareness/>
@@ -2489,98 +2471,13 @@ This may be set in a connection string with the C<w> option.
 The number of milliseconds an operation should wait for C<w> secondaries to
 replicate it.
 
-Defaults to 1000 (1 second).
+Defaults to 1000 (1 second). If you set this to undef, it could block indefinitely
+(or until socket timeout is reached).
 
 See C<w> above for more information.
 
 This may be set in a connection string with the C<wTimeoutMS> option.
 
-=head2 read_concern_level
-
-The read concern level determines the consistency level required
-of data being read.
-
-The default level is C<undef>, which means the server will use its configured
-default.
-
-If the level is set to "local", reads will return the latest data a server has
-locally.
-
-Additional levels are storage engine specific.  See L<Read
-Concern|http://docs.mongodb.org/manual/search/?query=readConcern> in the MongoDB
-documentation for more details.
-
-This may be set in a connection string with the the C<readConcernLevel> option.
-
-=head2 retry_writes
-
-Whether the client should use retryable writes for supported commands. The
-default value is false, which means that no write commands will be retried.
-
-If this is set to a true value, then commands which support retryable writes
-will be retried on certain errors, such as C<not master> and C<node is
-recovering> errors.
-
-This may be set in a connection string with the C<retryWrites> option.
-
-Note that this is only supported on MongoDB > 3.6 in Replica Set or Shard
-Clusters, and will be ignored on other deployments.
-
-Unacknowledged write operations also do not support retryable writes, even when
-retry_writes has been enabled.
-
-The supported single statement write operations are currently as follows:
-
-=over 4
-
-=item *
-
-C<insert_one>
-
-=item *
-
-C<update_one>
-
-=item *
-
-C<replace_one>
-
-=item *
-
-C<delete_one>
-
-=item *
-
-C<find_one_and_delete>
-
-=item *
-
-C<find_one_and_replace>
-
-=item *
-
-C<find_one_and_update>
-
-=back
-
-The supported multi statement write operations are as follows:
-
-=over 4
-
-=item *
-
-C<insert_many>
-
-=item *
-
-C<bulk_write>
-
-=back
-
-The multi statment operations may be ether ordered or unordered. Note that for
-C<bulk_write> operations, the request may not include update_many or
-delete_many operations.
-
 =head1 METHODS
 
 =head2 read_preference
@@ -2638,8 +2535,8 @@ Drops all connections to servers.
     $client->reconnect;
 
 This method closes all connections to the server, as if L</disconnect> were
-called, and then immediately reconnects.  Use this after forking or spawning
-off a new thread.
+called, and then immediately reconnects.  It also clears the session
+cache.  Use this after forking or spawning off a new thread.
 
 =head2 topology_status
 
@@ -2910,19 +2807,105 @@ The currently supported connection string options are:
 
 =over 4
 
-*appName
-*authMechanism
-*authMechanism.SERVICE_NAME
-*compressors
-*connectTimeoutMS
-*journal
-*readPreference
-*readPreferenceTags
-*replicaSet
-*ssl
-*w
-*wtimeoutMS
-*zlibCompressionLevel
+=item *
+
+C<appName>
+
+=item *
+
+C<authMechanism>
+
+=item *
+
+C<authMechanismProperties>
+
+=item *
+
+C<authSource>
+
+=item *
+
+C<compressors>
+
+=item *
+
+C<connect>
+
+=item *
+
+C<connectTimeoutMS>
+
+=item *
+
+C<heartbeatFrequencyMS>
+
+=item *
+
+C<journal>
+
+=item *
+
+C<localThresholdMS>
+
+=item *
+
+C<maxStalenessSeconds>
+
+=item *
+
+C<maxTimeMS>
+
+=item *
+
+C<readConcernLevel>
+
+=item *
+
+C<readPreference>
+
+=item *
+
+C<readPreferenceTags>
+
+=item *
+
+C<replicaSet>
+
+=item *
+
+C<retryWrites>
+
+=item *
+
+C<serverSelectionTimeoutMS>
+
+=item *
+
+C<serverSelectionTryOnce>
+
+=item *
+
+C<socketCheckIntervalMS>
+
+=item *
+
+C<socketTimeoutMS>
+
+=item *
+
+C<ssl>
+
+=item *
+
+C<w>
+
+=item *
+
+C<wTimeoutMS>
+
+=item *
+
+C<zlibCompressionLevel>
 
 =back
 
@@ -2933,13 +2916,13 @@ L<http://docs.mongodb.org/manual/reference/connection-string/>.
 =head1 SERVER SELECTION
 
 For a single server deployment or a direct connection to a mongod or
-mongos, all reads and writes and sent to that server.  Any read-preference
+mongos, all reads and writes are sent to that server.  Any read-preference
 is ignored.
 
 When connected to a deployment with multiple servers, such as a replica set
 or sharded cluster, the driver chooses a server for operations based on the
-type of operation (read or write), the types of servers available and a
-read preference.
+type of operation (read or write), application-provided server selector, the
+types of servers available and a read preference.
 
 For a replica set deployment, writes are sent to the primary (if available)
 and reads are sent to a server based on the L</read_preference> attribute,
@@ -2951,11 +2934,11 @@ servers in the seed list.  Any read preference is passed through to the
 mongos and used by it when executing reads against shards.
 
 If multiple servers can service an operation (e.g. multiple mongos servers,
-or multiple replica set members), one is chosen at random from within the
-"latency window".  The server with the shortest average round-trip time
-(RTT) is always in the window.  Any servers with an average round-trip time
-less than or equal to the shortest RTT plus the L</local_threshold_ms> are
-also in the latency window.
+or multiple replica set members), one is chosen by filtering with server
+selector and then at random from within the "latency window".  The server
+with the shortest average round-trip time (RTT) is always in the window.
+Any servers with an average round-trip time less than or equal to the
+shortest RTT plus the L</local_threshold_ms> are also in the latency window.
 
 If a suitable server is not immediately available, what happens next
 depends on the L</server_selection_try_once> option.
@@ -3064,7 +3047,7 @@ Usage information for each mechanism is given below.
 
 =head2 MONGODB-CR and SCRAM-SHA-1 (for username/password)
 
-These mechnisms require a username and password, given either as
+These mechanisms require a username and password, given either as
 constructor attributes or in the C<host> connection string.
 
 If a username is provided and an authentication mechanism is not specified,
@@ -3084,7 +3067,7 @@ fall back to MONGODB-CR for older servers.
 Usernames and passwords will be UTF-8 encoded before use.  The password is
 never sent over the wire -- only a secure digest is used.  The SCRAM-SHA-1
 mechanism is the Salted Challenge Response Authentication Mechanism
-definedin L<RFC 5802|http://tools.ietf.org/html/rfc5802>.
+defined in L<RFC 5802|http://tools.ietf.org/html/rfc5802>.
 
 The default database for authentication is 'admin'.  If another database
 name should be used, specify it with the C<db_name> attribute or via the
@@ -3109,6 +3092,10 @@ specifying the "MONGODB-X509" authentication mechanism.
         auth_mechanism => "MONGODB-X509",
     );
 
+B<Note>: Since MongoDB Perl driver v1.8.0, you no longer need to specify a
+C<username> parameter for X509 authentication; the username will be
+extracted automatically from the certificate.
+
 =head2 PLAIN (for LDAP)
 
 This mechanism requires a username and password, which will be UTF-8
@@ -3216,7 +3203,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2018 by MongoDB, Inc.
+This software is Copyright (c) 2019 by MongoDB, Inc.
 
 This is free software, licensed under:
 
