@@ -19,7 +19,7 @@ package MongoDB::Op::_Aggregate;
 # Encapsulate aggregate operation; return MongoDB::QueryResult
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
 
@@ -83,8 +83,10 @@ sub execute {
     # will reject it as unrecognized
     delete $options->{maxTimeMS} unless $is_2_6;
 
-    # bypassDocumentValidation isn't available until 3.2 (wire version 4)
-    delete $options->{bypassDocumentValidation} unless $link->supports_document_validation;
+    # bypassDocumentValidation isn't available until 3.2 (wire version 4) & dont send if false
+    unless ($link->supports_document_validation && $options->{bypassDocumentValidation}) {
+        delete $options->{bypassDocumentValidation};
+    }
 
     if ( defined $options->{collation} and !$link->supports_collation ) {
         MongoDB::UsageError->throw(
