@@ -20,7 +20,7 @@ package MongoDB::Op::_BulkWrite;
 # MongoDB::BulkWriteResult object
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
 
@@ -40,7 +40,6 @@ use Types::Standard qw(
     InstanceOf
 );
 use Safe::Isa;
-use Try::Tiny;
 use boolean;
 
 use namespace::clean;
@@ -227,14 +226,14 @@ sub _execute_write_command_batch {
             monitoring_callback => $self->monitoring_callback,
         );
 
-        my $cmd_result = try {
+        my $cmd_result = eval {
             $self->_is_retryable
               ? $self->client->send_retryable_write_op( $op )
               : $self->client->send_write_op( $op );
-        }
-        catch {
+        } or do {
+            my $error = $@ || "Unknown error";
             # This error never touches the database!.... so is before any retryable writes errors etc.
-            if ( $_->$_isa("MongoDB::_CommandSizeError") ) {
+            if ( $error->$_isa("MongoDB::_CommandSizeError") ) {
                 if ( @$chunk == 1 ) {
                     MongoDB::DocumentError->throw(
                         message  => "document too large",
@@ -242,13 +241,26 @@ sub _execute_write_command_batch {
                     );
                 }
                 else {
-                    unshift @left_to_send, $self->_split_chunk( $chunk, $_->size );
+                    unshift @left_to_send, $self->_split_chunk( $chunk, $error->size );
                 }
             }
+            elsif ( $error->$_can( 'result' ) ) {
+                # We are already going to explode from something here, but
+                # BulkWriteResult has the correct parsing method to allow us to
+                # check for write errors, as they have a higher priority than
+                # write concern errors.
+                MongoDB::BulkWriteResult->_parse_cmd_result(
+                    op => $type,
+                    op_count => scalar @$chunk,
+                    result => $error->result,
+                    cmd_doc => $cmd_doc,
+                )->assert_no_write_error;
+                # Explode with original error
+                die $error;
+            }
             else {
-                die $_;
+                die $error;
             }
-            return;
         };
 
         redo unless $cmd_result; # restart after a chunk split
@@ -400,16 +412,16 @@ sub _execute_legacy_batch {
             );
         }
 
-        my $op_result = try {
+        my $op_result = eval {
             $op->execute($link);
-        }
-        catch {
-            if (   $_->$_isa("MongoDB::DatabaseError")
-                && $_->result->does("MongoDB::Role::_WriteResult") )
+        } or do {
+            my $error = $@ || "Unknown error";
+            if (   $error->$_isa("MongoDB::DatabaseError")
+                && $error->result->does("MongoDB::Role::_WriteResult") )
             {
-                return $_->result;
+                return $error->result;
             }
-            die $_ unless $w_0 && /exceeds maximum size/;
+            die $error unless $w_0 && /exceeds maximum size/;
             return undef; ## no critic: this makes op_result undef
         };
 
