@@ -20,7 +20,7 @@ package MongoDB::Op::_ListIndexes;
 # Encapsulate index list operation; returns array ref of index documents
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
 
@@ -33,7 +33,6 @@ use Types::Standard qw(
   InstanceOf
 );
 use Tie::IxHash;
-use Try::Tiny;
 use Safe::Isa;
 
 use namespace::clean;
@@ -72,14 +71,14 @@ sub _command_list_indexes {
         monitoring_callback => $self->monitoring_callback,
     );
 
-    my $res = try {
+    my $res = eval {
         $op->execute( $link, $topology );
-    }
-    catch {
-        if ( $_->$_isa("MongoDB::DatabaseError") ) {
-            return undef if $_->code == NAMESPACE_NOT_FOUND(); ## no critic: make $res undef
+    } or do {
+        my $error = $@ || "Unknown error";
+        unless ( $error->$_isa("MongoDB::DatabaseError") and $error->code == NAMESPACE_NOT_FOUND()) {
+            die $error;
         }
-        die $_;
+        undef;
     };
 
     return $res
