@@ -19,7 +19,7 @@ package MongoDB::Role::_InsertPreEncoder;
 # MongoDB interface for pre-encoding and validating docs to insert
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo::Role;
 
@@ -45,6 +45,13 @@ sub _pre_encode_insert {
             $i < $#$doc ? $doc->[ $i + 1 ] : undef;
           }
         : $type eq 'Tie::IxHash' ? $doc->FETCH('_id')
+        : $type eq 'BSON::Raw' ? do {
+            my $decoded_doc = $self->bson_codec->decode_one(
+                $doc->bson,
+                { ordered => 1 }
+            );
+            $decoded_doc->{_id};
+            }
         : $doc->{_id} # hashlike?
     );
     if ( ! defined $id ) {
