@@ -19,21 +19,30 @@ package MongoDB::Role::_PrivateConstructor;
 # MongoDB interface for a private constructor
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
-
-use Moo::Role;
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use MongoDB::_Constants;
+use Sub::Defer;
 
-use namespace::clean;
+use Moo::Role;
 
 # When assertions are enabled, the private constructor delegates to the
 # public one, which checks required/isa assertions.  When disabled,
 # the private constructor blesses args directly to the class for speed.
 BEGIN {
+  my $NO_ASSERT_CONSTRUCTOR = <<'HERE';
+    my %done;
+    sub _new {
+      my $class = shift;
+      undefer_sub($class->can(q{new})) and $done{$class}++
+        unless $done{$class};
+      return bless {@_}, $class
+    }
+HERE
+
   WITH_ASSERTS
   ? eval 'sub _new { my $class = shift; $class->new(@_) }' ## no critic
-  : eval 'sub _new { my $class = shift; return bless {@_}, $class }'; ## no critic
+  : eval $NO_ASSERT_CONSTRUCTOR; ## no critic
 }
 
 1;
