@@ -19,16 +19,16 @@ use warnings;
 package MongoDB::_Protocol;
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use MongoDB::_Constants;
 use MongoDB::Error;
+use MongoDB::_Types qw/ to_IxHash /;
 
 use Compress::Zlib ();
 
 use constant {
     OP_REPLY        => 1,    # Reply to a client request. responseTo is set
-    OP_MSG          => 1000, # generic msg command followed by a string
     OP_UPDATE       => 2001, # update document
     OP_INSERT       => 2002, # insert new document
     RESERVED        => 2003, # formerly used for OP_GET_BY_OID
@@ -37,6 +37,7 @@ use constant {
     OP_DELETE       => 2006, # Delete documents
     OP_KILL_CURSORS => 2007, # Tell database client is done with a cursor
     OP_COMPRESSED   => 2012, # wire compression
+    OP_MSG          => 2013, # generic bi-directional op code
 };
 
 use constant {
@@ -64,6 +65,8 @@ use constant {
     P_KILL_CURSORS => PERL58 ? "l6(a8)*" : "l<6(a8)*",
     P_REPLY_HEADER => PERL58 ? "l5a8l2"  : "l<5a8l<2",
     P_COMPRESSED   => PERL58 ? "l6C"     : "l<6C",
+    P_MSG          => PERL58 ? "l5"      : "l<5",
+    P_MSG_PL_1     => PERL58 ? "lZ*"     : "l<Z*",
 };
 
 # struct MsgHeader {
@@ -86,8 +89,224 @@ use constant {
     # length for OP_COMPRESSED
     P_COMPRESSED_PREFIX_LENGTH =>
         length(pack P_COMPRESSED, 0, 0, 0, 0, 0, 0, 0),
+    P_MSG_PREFIX_LENGTH =>
+        length(pack P_MSG, 0, 0, 0, 0, 0),
 };
 
+# struct OP_MSG {
+#     MsgHeader header;             // standard message header, with opCode 2013
+#     uint32    flagBits;
+#     Section+  sections;
+#     [uint32   checksum;]
+# };
+#
+# struct Section {
+#     uint8 payloadType;
+#     union payload {
+#         document document;        // payloadType == 0
+#         struct   sequence {       // payloadType == 1
+#             int32     size;
+#             cstring   identifier;
+#             document* documents;
+#         };
+#     };
+# };
+
+use constant {
+    P_SECTION_PAYLOAD_TYPE  => "C",
+    P_SECTION_SEQUENCE_SIZE => PERL58 ? "l" : "l<",
+};
+
+use constant {
+    P_SECTION_HEADER               => P_SECTION_PAYLOAD_TYPE . P_SECTION_SEQUENCE_SIZE,
+    P_SECTION_PAYLOAD_TYPE_LENGTH  => length( pack P_SECTION_PAYLOAD_TYPE, 0 ),
+    P_SECTION_SEQUENCE_SIZE_LENGTH => length( pack P_SECTION_SEQUENCE_SIZE, 0 ),
+};
+
+# Takes a command, returns sections ready for joining
+
+sub prepare_sections {
+  my ( $codec, $cmd ) = @_;
+
+  my %split_commands = (
+    insert => 'documents',
+    update => 'updates',
+    delete => 'deletes',
+  );
+
+  $cmd = to_IxHash( $cmd );
+
+  # Command is always first key in cmd
+  my $command = do { my @keys = $cmd->Keys; $keys[0] };
+  my $ident = $split_commands{ $command };
+
+  if ( defined $ident ) {
+    my $collection = $cmd->FETCH( $command );
+    my $docs = $cmd->FETCH( $ident );
+    # Assumes only a single split on the commands
+    return (
+        {
+            type => 0,
+            documents => [ [
+                # Done specifically to not alter $cmd.
+                # The command ($command from earlier) is assumed to be
+                # first in the Keys set
+                map { $_ eq $ident
+                    ? ()
+                    : ( $_, $cmd->FETCH( $_ ) )
+                } $cmd->Keys()
+            ] ],
+        },
+        {
+            type => 1,
+            identifier => $ident,
+            documents => $docs,
+        }
+    );
+  } else {
+    # Not a recognised command to split, just set up ready for later
+    return (
+        {
+            type => 0,
+            documents => [ $cmd ],
+        }
+    );
+  }
+}
+
+# encode_section
+#
+#     MongoDB::_Protocol::encode_section( $codec, {
+#         type => 0,                  # 0 or 1
+#         identifier => undef,        # optional in type 0
+#         documents => [ $cmd ]       # must be an array of documents
+#     });
+#
+# Takes a section hashref and encodes it for joining
+
+sub encode_section {
+    my ( $codec, $section ) = @_;
+
+    my $type = $section->{type};
+    my $ident = $section->{identifier};
+    my @docs = map { $codec->encode_one( $_ ) } @{ $section->{documents} };
+
+    my $pl;
+    if ( $type == 0 ) {
+        # Assume a single doc if payload type is 0
+        $pl = $docs[0];
+    } elsif ( $type == 1 ) {
+        $pl = pack( P_MSG_PL_1, 0, $ident )
+          . join( '', @docs );
+        # calculate size
+        substr( $pl, 0, 4, pack( P_SECTION_SEQUENCE_SIZE, length( $pl ) ) );
+    } else {
+      MongoDB::ProtocolError->throw("Encode: Unsupported section payload type");
+    }
+
+    # Prepend the section type
+    $pl = pack( P_SECTION_PAYLOAD_TYPE, $type ) . $pl;
+
+    return $pl;
+}
+
+# decode_section
+#
+#     MongoDB::_Protocol::decode_section( $section )
+#
+# Takes an encoded section and decodes it, exactly the opposite of encode_section.
+
+sub decode_section {
+    my ( $doc ) = @_;
+    my ( $type, $ident, @enc_docs );
+    my $section = {};
+
+    ( $type ) = unpack( 'C', $doc );
+    my $payload = substr( $doc, P_SECTION_PAYLOAD_TYPE_LENGTH );
+
+    $section->{ type } = $type;
+
+    # Pull size off and double check. Size is in the same place regardless of
+    # payload type, as its a similar struct to a raw document
+    my ( $pl_size ) = unpack( P_SECTION_SEQUENCE_SIZE, $payload );
+    unless ( $pl_size == length( $payload ) ) {
+      MongoDB::ProtocolError->throw("Decode: Section size incorrect");
+    }
+
+    if ( $type == 0 ) {
+        # payload is a raw document
+        push @enc_docs, $payload;
+    } elsif ( $type == 1 ) {
+        $payload = substr( $payload, P_SECTION_SEQUENCE_SIZE_LENGTH );
+        # Pull out then remove
+        ( $ident ) = unpack( 'Z*', $payload );
+        $section->{ identifier } = $ident;
+        $payload = substr( $payload, length ( $ident ) + 1 ); # add one for null termination
+
+        while ( length $payload ) {
+          my $doc_size = unpack( P_SECTION_SEQUENCE_SIZE, $payload );
+          my $doc = substr( $payload, 0, $doc_size );
+          $payload = substr( $payload, $doc_size );
+          push @enc_docs, $doc;
+        }
+    } else {
+        MongoDB::ProtocolError->throw("Decode: Unsupported section payload type");
+    }
+    $section->{ documents } = \@enc_docs;
+
+    return $section;
+}
+
+# method split_sections( $msg )
+#
+# Splits sections based on their payload length header. Returns an array of
+# sections in packed form
+
+sub split_sections {
+  my $msg = shift;
+  my @sections;
+  while ( length $msg ) {
+    # get first section length
+    my ( undef, $section_length ) = unpack( P_SECTION_HEADER, $msg );
+
+    # Add the payload type length as we reached over it for the length
+    my $section = substr( $msg, 0, $section_length + P_SECTION_PAYLOAD_TYPE_LENGTH );
+
+    push @sections, decode_section( $section );
+
+    $msg = substr( $msg, $section_length + P_SECTION_PAYLOAD_TYPE_LENGTH );
+  }
+
+  return @sections;
+}
+
+use constant {
+  MSG_FB_CHECKSUM => 0,
+  MSG_FB_MORE_TO_COME => 1,
+};
+
+sub write_msg {
+  my ( $codec, $flags, $cmd ) = @_;
+  my $flagbits = 0;
+  # checksum is reserved for future use
+  if ( $flags ) {
+    $flagbits =
+        ( $flags->{checksum_present} ? 1 << MSG_FB_CHECKSUM     : 0 )
+      | ( $flags->{more_to_come}     ? 1 << MSG_FB_MORE_TO_COME : 0 );
+  }
+
+  my $request_id = int( rand( MAX_REQUEST_ID ) );
+
+  my @sections = prepare_sections( $codec, $cmd );
+
+  my $encoded_sections = join ('', ( map { encode_section( $codec, $_ ) } @sections ) );
+
+  my $msg = pack( P_MSG, 0, $request_id, 0, OP_MSG, 0 )
+    . $encoded_sections;
+  substr( $msg, 0, 4, pack( P_INT32, length($msg) ) );
+  return ( $msg, $request_id );
+}
+
 # struct OP_COMPRESSED {
 #     MsgHeader header;             // standard message header
 #     int32_t   originalOpcode;     // wrapped op code
@@ -125,7 +344,7 @@ sub get_compressor {
             callback => sub {
                 return Compress::Zlib::compress(
                     $_[0],
-                    defined($level) ? $level : (),
+                    defined($level) ? $level : Compress::Zlib::Z_DEFAULT_COMPRESSION(),
                 );
             },
         };
@@ -204,8 +423,8 @@ sub write_update {
     my $bitflags = 0;
     if ($flags) {
         $bitflags =
-          ( $flags->{upsert} ? 1 << U_UPSERT : 0 )
-          | ( $flags->{multi} ? 1 << U_MULTI_UPDATE : 0 );
+            ( $flags->{upsert} ? 1 << U_UPSERT       : 0 )
+          | ( $flags->{multi}  ? 1 << U_MULTI_UPDATE : 0 );
     }
 
     my $msg =
@@ -390,7 +609,6 @@ use constant {
 
 sub parse_reply {
     my ( $msg, $request_id ) = @_;
-
     MongoDB::ProtocolError->throw("response was truncated")
         if length($msg) < MIN_REPLY_LENGTH;
 
@@ -399,7 +617,7 @@ sub parse_reply {
     my (
         $len, $msg_id, $response_to, $opcode, $bitflags, $cursor_id, $starting_from,
         $number_returned
-    ) = unpack( P_REPLY_HEADER, $msg );
+    ) = unpack( P_MSG, $msg );
 
     # pre-check all conditions using a modifier in one statement for speed;
     # disambiguate afterwards only if an error exists
@@ -410,8 +628,8 @@ sub parse_reply {
             MongoDB::ProtocolError->throw("response was truncated");
         }
 
-        if ( $opcode != OP_REPLY ) {
-            MongoDB::ProtocolError->throw("response was not OP_REPLY");
+        if ( $opcode != OP_REPLY && $opcode != OP_MSG ) {
+            MongoDB::ProtocolError->throw("response was not OP_REPLY or OP_MSG");
         }
 
         if ( $response_to != $request_id ) {
@@ -420,9 +638,30 @@ sub parse_reply {
         }
         }
         if ( length($msg) < $len )
-        || ( $opcode != OP_REPLY )
+        || ( ( $opcode != OP_REPLY ) && ( $opcode != OP_MSG ) )
         || ( $response_to != $request_id );
 
+
+    if ( $opcode == OP_MSG ) {
+        # XXX Extract and check checksum - future support of crc32c
+        my @sections = split_sections( substr( $msg, P_MSG_PREFIX_LENGTH ) );
+        # We have none of the other stuff? maybe flags... and an array of docs? erm
+        return {
+          flags => {
+            checksum_present => vec( $bitflags, MSG_FB_CHECKSUM, 1 ),
+            more_to_come    => vec( $bitflags, MSG_FB_MORE_TO_COME, 1 ),
+          },
+          # XXX Assumes the server never sends a type 1 payload. May change in future
+          docs => $sections[0]->{documents}->[0]
+        };
+    } else {
+        # Yes its two unpacks but its just easier than mapping through to the right size
+        (
+            $len, $msg_id, $response_to, $opcode, $bitflags, $cursor_id, $starting_from,
+            $number_returned
+        ) = unpack( P_REPLY_HEADER, $msg );
+    }
+
     # returns non-zero cursor_id as blessed object to identify it as an
     # 8-byte opaque ID rather than an ambiguous Perl scalar. N.B. cursors
     # from commands are handled differently: they are perl integers or
