@@ -17,7 +17,7 @@ use warnings;
 package MongoDB::_SessionPool;
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
 use MongoDB::_ServerSession;
@@ -26,20 +26,31 @@ use Types::Standard qw(
     InstanceOf
 );
 
-has client => (
+has dispatcher => (
     is => 'ro',
     required => 1,
-    weak_ref => 1,
-    isa => InstanceOf['MongoDB::MongoClient'],
+    isa => InstanceOf['MongoDB::_Dispatcher'],
+);
+
+has topology=> (
+    is => 'ro',
+    required => 1,
+    isa => InstanceOf['MongoDB::_Topology'],
 );
 
 has _server_session_pool => (
     is => 'lazy',
     isa => ArrayRef[InstanceOf['MongoDB::_ServerSession']],
     init_arg => undef,
+    clearer => 1,
     builder => sub { [] },
 );
 
+has _pool_epoch => (
+    is => 'rwp',
+    init_arg => undef,
+    default => 0,
+);
 
 # Returns a L<MongoDB::ServerSession> that was at least one minute remaining
 # before session times out. Returns undef if no sessions available.
@@ -50,14 +61,14 @@ sub get_server_session {
     my ( $self ) = @_;
 
     if ( scalar( @{ $self->_server_session_pool } ) > 0 ) {
-        my $session_timeout = $self->client->_topology->logical_session_timeout_minutes;
+        my $session_timeout = $self->topology->logical_session_timeout_minutes;
         # if undefined, sessions not actually supported so drop out here
         while ( my $session = shift @{ $self->_server_session_pool } ) {
             next if $session->_is_expiring( $session_timeout );
             return $session;
         }
     }
-    return MongoDB::_ServerSession->new;
+    return MongoDB::_ServerSession->new( pool_epoch => $self->_pool_epoch );
 }
 
 # Place a session back into the pool for use. Will check that there is at least
@@ -70,7 +81,9 @@ sub get_server_session {
 sub retire_server_session {
     my ( $self, $server_session ) = @_;
 
-    my $session_timeout = $self->client->_topology->logical_session_timeout_minutes;
+    return if $server_session->pool_epoch != $self->_pool_epoch;
+
+    my $session_timeout = $self->topology->logical_session_timeout_minutes;
 
     # Expire old sessions from back of queue
     while ( my $session = $self->_server_session_pool->[-1] ) {
@@ -96,19 +109,26 @@ sub end_all_sessions {
 
     for my $batch ( @batches ) {
         my $sessions = [
-            # TODO For some reason, the sessions are getting demolished before
-            # here, even though we have references to them...
             map { defined $_ ? $_->session_id : () } @$batch
         ];
         # Ignore any errors generated from this
         eval {
-            $self->client->send_admin_command([
+            $self->dispatcher->send_admin_command([
                 endSessions => $sessions,
             ], 'primaryPreferred');
         };
     }
 }
 
+# When reconnecting a client after a fork, we need to clear the pool
+# without ending sessions with the server and increment the pool epoch
+# so existing sessions aren't checked back in.
+sub reset_pool {
+    my ( $self ) = @_;
+    $self->_clear_server_session_pool;
+    $self->_set__pool_epoch( $self->_pool_epoch + 1 );
+}
+
 sub DEMOLISH {
     my ( $self, $in_global_destruction ) = @_;
 
