@@ -17,7 +17,7 @@ use warnings;
 package MongoDB::_Topology;
 
 use version;
-our $VERSION = 'v1.999.1'; # TRIAL
+our $VERSION = 'v2.1.0'; # TRIAL
 
 use Moo;
 use BSON;
@@ -52,7 +52,6 @@ use Config;
 use List::Util qw/first max min/;
 use Safe::Isa;
 use Time::HiRes qw/time usleep/;
-use Try::Tiny;
 
 use namespace::clean;
 
@@ -181,6 +180,11 @@ has server_selection_try_once => (
     isa => Boolish,
 );
 
+has server_selector => (
+    is => 'ro',
+    isa => Maybe[CodeRef],
+);
+
 has ewma_alpha => (
     is      => 'ro',
     default => 0.2,
@@ -597,6 +601,8 @@ sub _check_wire_versions {
         my ( $server_min_wire_version, $server_max_wire_version ) =
           @{ $server->is_master }{qw/minWireVersion maxWireVersion/};
 
+        # set to 0 as could be undefined. 0 is the equivalent to missing, and
+        # also kept as 0 for legacy compatibility.
         $server_max_wire_version = 0 unless defined $server_max_wire_version;
         $server_min_wire_version = 0 unless defined $server_min_wire_version;
 
@@ -645,21 +651,6 @@ sub _supports_sessions {
     return defined $self->logical_session_timeout_minutes;
 }
 
-# Used for bulkWrite for shortcutting to original execute command
-sub _supports_retry_writes {
-    my ( $self ) = @_;
-
-    # retryWrites arent supported in standalone servers
-    return if $self->type eq 'Single';
-
-    # retryWrites require a wire version of at least six
-    return if $self->wire_version_ceil < 6;
-
-    # must have lstm present
-    return 1 if defined $self->logical_session_timeout_minutes;
-    return;
-}
-
 sub _supports_transactions {
     my ( $self ) = @_;
 
@@ -770,8 +761,11 @@ sub _find_available_server {
     my ( $self, $read_pref, @candidates ) = @_;
     $self->_check_staleness_compatibility($read_pref) if $read_pref;
     push @candidates, $self->all_servers unless @candidates;
+    my $selector = $self->server_selector;
     return $self->_get_server_in_latency_window(
-        [ grep { $_->is_available } @candidates ] );
+      [ grep { $_->is_available }
+          $selector ? $selector->(@candidates) : @candidates ]
+    );
 }
 
 # This uses read preference to check for max staleness compatibility in
@@ -780,8 +774,11 @@ sub _find_readable_mongos_server {
     my ( $self, $read_pref, @candidates ) = @_;
     $self->_check_staleness_compatibility($read_pref);
     push @candidates, $self->all_servers unless @candidates;
+    my $selector = $self->server_selector;
     return $self->_get_server_in_latency_window(
-        [ grep { $_->is_available } @candidates ] );
+      [ grep { $_->is_available }
+          $selector ? $selector->(@candidates) : @candidates ]
+    );
 }
 
 sub _find_nearest_server {
@@ -789,7 +786,10 @@ sub _find_nearest_server {
     $self->_check_staleness_compatibility($read_pref);
     push @candidates, ( $self->_primaries, $self->_secondaries ) unless @candidates;
     my @suitable = $self->_eligible( $read_pref, @candidates );
-    return $self->_get_server_in_latency_window( \@suitable );
+    my $selector = $self->server_selector;
+    return $self->_get_server_in_latency_window(
+        [ $selector ? $selector->(@suitable) : @suitable ]
+    );
 }
 
 sub _find_primary_server {
@@ -812,7 +812,10 @@ sub _find_secondary_server {
     $self->_check_staleness_compatibility($read_pref);
     push @candidates, $self->_secondaries unless @candidates;
     my @suitable = $self->_eligible( $read_pref, @candidates );
-    return $self->_get_server_in_latency_window( \@suitable );
+    my $selector = $self->server_selector;
+    return $self->_get_server_in_latency_window(
+        [ $selector ? $selector->(@suitable) : @suitable ]
+    );
 }
 
 sub _find_secondarypreferred_server {
@@ -842,6 +845,25 @@ sub _get_server_in_latency_window {
     return $in_window[ int( rand(@in_window) ) ]->{server};
 }
 
+my $PRIMARY = MongoDB::ReadPreference->new;
+my $PRIMARY_PREF = MongoDB::ReadPreference->new( mode => 'primaryPreferred' );
+
+sub _ping_server {
+    my ($self, $link) = @_;
+    return eval {
+        my $op = MongoDB::Op::_Command->_new(
+            db_name             => 'admin',
+            query               => [ping => 1],
+            query_flags         => {},
+            bson_codec          => $self->bson_codec,
+            read_preference     => $PRIMARY_PREF,
+            monitoring_callback => $self->monitoring_callback,
+        );
+        $op->execute( $link )->output;
+    };
+}
+
+
 sub _get_server_link {
     my ( $self, $server, $method, $read_pref ) = @_;
     my $address = $server->address;
@@ -853,6 +875,10 @@ sub _get_server_link {
 
     # for idle links, refresh the server and verify validity
     if ( time - $link->last_used > $self->socket_check_interval_sec ) {
+        return $link if $self->_ping_server;
+        $self->mark_server_unknown(
+          $server, 'Lost connection with the server'
+        );
         $self->check_address($address);
 
         # topology might have dropped the server
@@ -872,13 +898,12 @@ sub _get_server_link {
 sub _initialize_link {
     my ( $self, $address ) = @_;
 
-    my $link = try {
+    my $link = eval {
         MongoDB::_Link->new( %{$self->link_options}, address => $address )->connect;
-    }
-    catch {
+    } or do {
+        my $error = $@ || "Unknown error";
         # if connection failed, update topology with Unknown description
-        $self->_reset_address_to_unknown( $address, $_ );
-        return;
+        $self->_reset_address_to_unknown( $address, $error );
     };
 
     return unless $link;
@@ -895,11 +920,11 @@ sub _initialize_link {
     # try to authenticate; if authentication fails, all
     # servers are considered invalid and we throw an error
     if ( $self->type eq 'Single' || first { $_ eq $server->type } qw/Standalone Mongos RSPrimary RSSecondary/ ) {
-        try {
+        eval {
             $self->credential->authenticate($server, $link, $self->bson_codec);
-        }
-        catch {
-            my $err = $_;
+            1;
+        } or do {
+            my $err = $@;
             my $msg = $err->$_isa("MongoDB::Error") ? $err->message : "$err";
             $self->_reset_address_to_unknown( $_->address, $err ) for $self->all_servers;
             MongoDB::AuthError->throw("Authentication to $address failed: $msg");
@@ -1032,8 +1057,6 @@ sub _selection_timeout {
     }
 }
 
-my $PRIMARY = MongoDB::ReadPreference->new;
-
 sub _generate_ismaster_request {
     my ( $self, $link, $should_perform_handshake ) = @_;
     my @opts;
@@ -1200,8 +1223,9 @@ sub _update_rs_with_primary_from_member {
         $self->_remove_server($new_server);
     }
 
-    # require 'me' that matches expected address
-    if ( $new_server->me && $new_server->me ne $new_server->address ) {
+    # require 'me' that matches expected address.
+    # check is case insensitive
+    if ( $new_server->me && lc $new_server->me ne $new_server->address ) {
         $self->_remove_server($new_server);
         $self->_check_for_primary;
         return;
